package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.TemplateDto;
import com.qmth.distributed.print.business.bean.params.TemplateParam;
import com.qmth.distributed.print.business.entity.BasicTemplate;
import com.qmth.distributed.print.business.entity.SysOrg;
import com.qmth.distributed.print.business.mapper.BasicTemplateMapper;
import com.qmth.distributed.print.business.service.BasicTemplateOrgService;
import com.qmth.distributed.print.business.service.BasicTemplateService;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 通用模板 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicTemplateServiceImpl extends ServiceImpl<BasicTemplateMapper, BasicTemplate> implements BasicTemplateService {

    @Autowired
    private BasicTemplateOrgService basicTemplateOrgService;

    @Override
    public IPage<TemplateDto> list(Long schoolId, Boolean enable, String type, String name, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Page<TemplateDto> page = new Page<>(pageNumber, pageSize);
        IPage<TemplateDto> templateDtoIPage = this.baseMapper.listPage(page, schoolId, enable, type, name, startTime, endTime);
        if (templateDtoIPage.getRecords().size() > 0) {
            templateDtoIPage.getRecords().forEach(m -> {
                //查询适用学院
                List<SysOrg> sysOrgs = basicTemplateOrgService.listByTypeAndTemplateId(type, Long.valueOf(m.getId()));
                m.setOrgs(sysOrgs);
            });
        }
        return templateDtoIPage;
    }

    @Override
    public boolean saveTemplate(BasicTemplate template) {
        // todo 校验是否有必选字段已使用
        QueryWrapper<BasicTemplate> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicTemplate::getSchoolId, template.getSchoolId()).eq(BasicTemplate::getName, template.getName());
        BasicTemplate basicTemplate = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(template.getId())) {
            if (basicTemplate != null) {
                throw ExceptionResultEnum.ERROR.exception("模板名称已存在");
            }
            template.setId(SystemConstant.getDbUuid());
            template.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(template);
        }
        // 修改
        else {
            if (basicTemplate != null && basicTemplate.getId() != template.getId()) {
                throw ExceptionResultEnum.ERROR.exception("模板名称已存在");
            }

            template.setUpdateTime(System.currentTimeMillis());

            //删除权限
            basicTemplateOrgService.removeByRuleId(template.getId());
            isSuccess = this.updateById(template);

        }

        // 新增权限
        basicTemplateOrgService.saveBatch(template.getType(), template.getId(), template.getOrgIds(), template.getCreateId());

        return isSuccess;
    }

    @Override
    public boolean enable(BasicTemplate template) {
        UpdateWrapper<BasicTemplate> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicTemplate::getEnable, template.getEnable()).eq(BasicTemplate::getId, template.getId());
        return this.update(updateWrapper);
    }
}
