package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.bean.params.ExamCardParams;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.mapper.ExamTaskDetailMapper;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 命题任务明细表（入库申请） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskDetailServiceImpl extends ServiceImpl<ExamTaskDetailMapper, ExamTaskDetail> implements ExamTaskDetailService {

    @Autowired
    private ExamTaskService examTaskService;

    @Override
    public boolean enable(ExamTaskDetail examTaskDetail) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getEnable, examTaskDetail.getEnable()).eq(ExamTaskDetail::getId, examTaskDetail.getId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean updatePaper(Map<String, String> map) {
        // 更新试卷编号
        UpdateWrapper<ExamTask> examTaskUpdateWrapper = new UpdateWrapper<>();
        examTaskUpdateWrapper.lambda().set(ExamTask::getPaperNumber, map.get("paperNumber"));
        examTaskService.update(examTaskUpdateWrapper);

        // 更新关联卷型
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getRelatePaperType, map.get("relatePaperType")).eq(ExamTaskDetail::getExamTaskId, map.get("examTaskId"));
        this.update(updateWrapper);

        // todo 更新卷型曝光类型
        return true;
    }

    @Override
    public IPage<ExamTaskDetailDto> list(Long schoolId, String relateType, Long printPlanId, String courseCode, String paperNumber, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examDetailDtoIPage = this.baseMapper.listPage(page, schoolId, relateType, printPlanId, courseCode, paperNumber);
        return examDetailDtoIPage;
    }

    @Override
    public List<ExamTaskDetail> listByCardId(Long cardId) {
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getCardId, cardId);
        return this.list(queryWrapper);
    }

    @Override
    public void bindCardId(String examTaskId, Long cardId) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getCardId, cardId).eq(ExamTaskDetail::getExamTaskId, examTaskId);
        this.update(updateWrapper);
    }

    @Override
    public void resetCardId(Long id) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getEnable, null).eq(ExamTaskDetail::getExamTaskId, id);
        this.update(updateWrapper);
    }

}
